package mdd.client;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.zeromq.SocketType;
import org.zeromq.ZMQ;
import org.zeromq.ZContext;

public class Client implements IGUIEventClient
{
    private ZContext context;
    private ZMQ.Socket req_socket;
    private ZMQ.Socket sub_socket;
    private long server_id;
    private long msg_counter = 0;
    private Thread listenerThread;
    private GUIEventHandler _eventHandler;

    private void request_state(){
        JSONObject json = new JSONObject();
        json.put("state", "all");
        JSONObject jmsg = request("get",json);
        server_id = (long)jmsg.get("serverID");
        msg_counter = (long) jmsg.get("msgNr");
        System.out.println("TEST");
    }

    private void listen(){
        while (!Thread.currentThread().isInterrupted()) {
            // Read envelope with address
            String address = sub_socket.recvStr();
            // Read message contents
            String content = sub_socket.recvStr();

            JSONParser jsonParser = new JSONParser();
            JSONObject jmsg = null;
            try {
                jmsg = (JSONObject) jsonParser.parse(content);
            } catch (ParseException e) {
                e.printStackTrace();
            }

            if ((long) jmsg.get("serverID") == server_id){
                if ((long) jmsg.get("msgNr") == msg_counter) {
                    _eventHandler.publishEvent("change", jmsg);
                    System.out.println("[Client] "+ "change: " + jmsg.toString());
                    ++msg_counter;
                } else {
                    request_state();
                }
            }
            System.out.println(address + " : " + content);
        }
    }

    public Client(){
        _eventHandler = GUIEventHandler.getEventHandler();
        _eventHandler.addEventListener(this);
        try{
            context = new ZContext();
            req_socket = context.createSocket(SocketType.REQ);
            req_socket.connect("tcp://localhost:5555");

            sub_socket = context.createSocket(SocketType.SUB);
            sub_socket.connect("tcp://localhost:5556");
            sub_socket.subscribe("CHANGE".getBytes(ZMQ.CHARSET));
        } catch (Exception e) {
            e.printStackTrace();
        }
        request_state();

        Runnable runnable = this::listen;
        listenerThread = new Thread(runnable);
        listenerThread.start();
    }

    public JSONObject request(JSONObject json){
        req_socket.send(json.toString().getBytes(ZMQ.CHARSET), 0);
        String reply = req_socket.recvStr(0);
        JSONParser jsonParser = new JSONParser();
        JSONObject jmsg = null;
        try {
            jmsg = (JSONObject) jsonParser.parse(reply);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return jmsg;
    }

    public JSONObject request(String flag, JSONObject json){
        JSONObject args = new JSONObject();
        args.put(flag, json);
        return request(args);
    }

    @Override
    public void processGUIEvent(String event, JSONObject args) {
        if (event.equals("try")){
            request("try",args);
        }
        if (event.equals("get")){
            _eventHandler.publishEvent("receive", request("get", args));
        }
    }

    @Override
    public void close() throws Exception {
        listenerThread.join();
        //close Server
        _eventHandler.removeEventListener(this);
    }
}