package mdd.client;


import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import org.json.simple.JSONObject;

import java.util.Vector;


public class Module extends Group implements IGUIEventClient{
    private boolean _dragged = false;
    private Point2D _pos = new Point2D(0.0f,0.0f);
    protected GUIEventHandler _eventHandler;

    public String ID = "";
    public Vector<String> Prefix;
    protected String Type = "module";
    public HBox hlayout = new HBox();
    public VBox inputs = new VBox();
    public VBox outputs = new VBox();
    public boolean isSelected = false;

    private void init(Color color){
        setFocusTraversable(true);
        _eventHandler = GUIEventHandler.getEventHandler();
        _eventHandler.addEventListener(this);

        inputs.setSpacing(10);

        outputs.setSpacing(10);


        hlayout.getChildren().addAll(inputs, outputs);
        hlayout.setSpacing(20);
        hlayout.setTranslateX(-7);
        hlayout.setTranslateY(30);

        // System.out.println("[Module]: pressed: "+ hbox.getPrefWidth());
        Rectangle titlebar;
        Rectangle background;
        Text caption;

        titlebar = new Rectangle(0.0f, 0.0f, 150-20, 20.0f);
        titlebar.setArcWidth(10.0f);
        titlebar.setArcHeight(10.0f);
        titlebar.setFill(color);
        background = new Rectangle(0.0f, 0.0f, 150-20, 150 + 40);
        background.setArcWidth(10.0f);
        background.setArcHeight(10.0f);
        background.setFill(Color.GRAY);
        caption = new Text("TEST");
        caption.setFont(new Font("Arial", 15));
        caption.getStyleClass().add("text");
        caption.setX(10);
        caption.setY(15);


        getChildren().addAll( background, titlebar, caption, hlayout);



        addEventFilter(MouseEvent.MOUSE_PRESSED, e -> {
            if(e.getButton() == MouseButton.PRIMARY) {
                isSelected = true;
                System.out.println("[Module]: select: " + ID);
                _eventHandler.publishEvent("MOUSE_PRESSED", getJSONIdentifier());
                _pos = new Point2D(e.getSceneX(), e.getSceneY());
            }
            if(e.getButton() == MouseButton.SECONDARY) {

            }
            //e.consume();
        });

        addEventHandler(MouseEvent.MOUSE_DRAGGED, e -> {
            // System.out.println("[Module]: dragg event detected!");
            if(e.getButton() == MouseButton.PRIMARY) {
                if (isSelected){
                    setTranslateX(getTranslateX()+(e.getSceneX()-_pos.getX()));
                    setTranslateY(getTranslateY()+(e.getSceneY()-_pos.getY()));
                    _pos = new Point2D( e.getSceneX(),  e.getSceneY());
                }
            }
            if(e.getButton() == MouseButton.SECONDARY) {

            }
        });
    }

    public static Module DefaultModule(){
        return new Module(new Vector<String>(), "", Color.DARKBLUE);
    }


    public Module(JSONObject json){
        init(Color.DARKCYAN);
    }

    public Module(Vector<String> prefix, String id, Color color){
        Prefix = prefix;
        ID = id;

        addInput(new Input(this,"1"));
        addInput(new Input(this,"2"));
        addInput(new Input(this,"3"));

        addOutput(new Output(this,"1"));
        addOutput(new Output(this,"2"));
        addOutput(new Output(this,"3"));

        init(color);
    }

    public void addInput(Input in){
        inputs.getChildren().add(in);
    }

    public void addOutput(Output out){
        outputs.getChildren().add(out);
    }

    public Input getInput(String id){
        for (Node node : inputs.getChildren()){
            if(node instanceof Input){
                Input in = (Input) node;
                if (in.ID.equals(id)){
                    return in;
                }
            }

        }
        System.out.println("[Module]: getInput: WRONG ID!");
        return null;
    }

    public Output getOutput(String id){
        for (Node node : outputs.getChildren()){
            if(node instanceof Output){
                Output out = (Output) node;
                if (out.ID.equals(id)){
                    return out;
                }
            }

        }
        System.out.println("[Module]: getOutput: WRONG ID!");
        return null;
    }

    public JSONObject getJSONIdentifier(){
        JSONObject json = new JSONObject();
        json.put("prefix", Prefix);
        json.put("id", ID);
        json.put("type", Type);
        return json;
    }

    public Vector<String> getAsPrefix(){
        Vector<String> ret = (Vector<String>)Prefix.clone();
        ret.add(ID);
        return ret;
    }

    @Override
    public void processGUIEvent(String event, JSONObject args) {
        if (event == "MOUSE_PRESSED"){


            if (!args.equals(getJSONIdentifier())){
                if (isSelected){
                    System.out.println("[Module]: deselect: " + ID);
                }
                isSelected = false;
            }
        }
    }

    @Override
    public void close() throws Exception {
        _eventHandler.removeEventListener(this);
    }
}
