package mdd.client;

import javafx.event.EventHandler;
import javafx.geometry.Point2D;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.input.*;
import javafx.scene.paint.Color;
import javafx.scene.shape.Line;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import java.util.Vector;

public class Processor extends Module{
    private Group modules = new Group();
    private Group connections = new Group();
    public Group all = new Group(connections, modules);


    private boolean detected_drag = false;
    private boolean draw_temp_connection = false;
    public Connection temp_connection = new Connection();
    private Connectable connectable;

    private void init(){
        Type = "processor";
        modules.addEventHandler(MouseEvent.DRAG_DETECTED, e -> {
            // System.out.println("[Module]: dragg event detected!");
            if(e.getButton() == MouseButton.PRIMARY) {
                detected_drag = true;
            }
            if(e.getButton() == MouseButton.SECONDARY) {

            }
        });

        all.addEventHandler(MouseEvent.MOUSE_DRAGGED, e -> {
            // System.out.println("[Module]: dragg event detected!");
            if(e.getButton() == MouseButton.PRIMARY) {
                if (draw_temp_connection){
                    System.out.println("[Processor] draw Line!");
                    temp_connection.setEndX(e.getSceneX());
                    temp_connection.setEndY(e.getSceneY());
                }
            }
            if(e.getButton() == MouseButton.SECONDARY) {

            }
        });

        all.addEventHandler(DragEvent.DRAG_DROPPED, e -> {
            //connections.getChildren().remove(temp_connection);
            System.out.println("[Processor] all");
            detected_drag = false;
        });

        modules.addEventHandler(DragEvent.DRAG_DROPPED, e -> {
            //connections.getChildren().remove(temp_connection);
            System.out.println("[Processor]");
            detected_drag = false;

            Dragboard db = e.getDragboard();
            boolean success = false;
            if (db.hasString()) {
                JSONParser parser = new JSONParser();
                JSONArray jsonList = null;
                try {
                    jsonList = (JSONArray)parser.parse(db.getString());
                } catch (ParseException parseException) {
                    parseException.printStackTrace();
                }

                assert jsonList != null;
                if (jsonList.size() == 2){

                    boolean foundInput = false;
                    boolean foundOutput = false;
                    Input in = new Input(this, "-1");
                    Output out = new Output(this, "-1");
                    for (Object o : jsonList) {
                        JSONObject json = (JSONObject) o;
                        if (json.containsKey("type")) {
                            String type = json.get("type").toString();
                            if (type.equals("INPUT")) {
                                System.out.println("[Processor] Dropped found type = " + "INPUT");
                                foundInput = true;
                                in = getModule(json.get("parent").toString()).getInput(json.get("ID").toString());
                            } else if (type.equals("OUTPUT")) {
                                System.out.println("[Processor] Dropped found type = " + "OUTPUT");
                                foundOutput = true;
                                out = getModule(json.get("parent").toString()).getOutput(json.get("ID").toString());
                            }
                        }
                    }
                    if (foundInput && foundOutput){
                        System.out.println("[Processor] Dropped: success!");
                        if(in.connected){
                            for (int i = 0; i < connections.getChildren().size(); ++i){
                                if (((Connection)connections.getChildren().get(i)).in.equals(in)){
                                    connections.getChildren().remove(i);
                                }
                            }
                        }
                        connections.getChildren().add(new Connection(in, out));
                        success = true;
                    }

                }

                System.out.println("[Processor] Dropped: " + db.getString());

            }
            e.setDropCompleted(success);
            e.consume();
        });
    }

    public Processor(JSONObject json){
        super(json);
        init();
    }

    public Processor(Vector<String> prefix, String id) {
        super(prefix, id, Color.DARKVIOLET);
        init();
    }

    public Module getModule(String id){
        for (Node node : modules.getChildren()){
            if (node instanceof Module){
                Module m = (Module)node;
                if (m.ID.equals(id)){
                    return m;
                }
            }
        }
        return null;
    }

    public Module getModule(Vector<String> id_path){
        id_path.remove(0);
        if (id_path.size() == 1){
            return getModule(id_path.firstElement());
        }
        else {
            Processor new_processor = (Processor)getModule(id_path.firstElement());
            return new_processor.getModule(id_path);
        }
    }

    public Processor getProcessor(Vector<String> id_path){
        id_path.remove(0);
        Processor new_processor = (Processor)getModule(id_path.firstElement());
        if (id_path.size() == 1){
            return new_processor;
        }else {
            return new_processor.getProcessor(id_path);
        }
    }

    public void addModule(Module module){
        JSONObject json = new JSONObject();
        json.put("id", getJSONIdentifier());
        json.put("add", module.getJSONIdentifier());
        _eventHandler.publishEvent("try", json);
        modules.getChildren().add(module);
    }

    public void addConnection(Connection connection){
        connections.getChildren().add(connection);
    }

    @Override
    public void processGUIEvent(String event, JSONObject args) {
        super.processGUIEvent(event, args);
        if (isSelected){
            if (event.equals("KeyTyped")){
                String key = args.get("key").toString();
                if (key.equals("ENTER")){
                    JSONObject json = new JSONObject();
                    json.put("path", getAsPrefix());
                    _eventHandler.publishEvent("OPEN_PROCESSOR", json);
                    System.out.println("[Processor]: published");
                }
            }
        }
        if (detected_drag){
            if (event.equals("DRAG_LINE")){
                draw_temp_connection = true;

                JSONObject json = args;
                if (json.containsKey("type")) {
                    String type = json.get("type").toString();
                    if (type.equals("INPUT")) {
                        System.out.println("[Processor] Line from type = " + "INPUT");
                        connectable = getModule(json.get("parent").toString()).getInput(json.get("ID").toString());
                    } else if (type.equals("OUTPUT")) {
                        System.out.println("[Processor] Line from type = " + "OUTPUT");
                        connectable = getModule(json.get("parent").toString()).getOutput(json.get("ID").toString());
                    }
                    if (connectable == null){
                        System.out.println("[Processor]: Couldn't find Connectable!");
                    }
                }
                temp_connection= new Connection();
                temp_connection.startXProperty().bind(connectable.x);
                temp_connection.startYProperty().bind(connectable.y);
                temp_connection.setEndX(connectable.x.getValue()+100);
                temp_connection.setEndY(connectable.y.getValue()+100);
                //connections.getChildren().add(temp_connection);
            }
        }
        if (event.equals("change")){
            /*
            "{
            "id":{...},
            "add":
                {

                }
            }"
             */
            if (args.containsKey("id")){
                System.out.println("[Processor] is " + args.get("id").toString() + " == " + getJSONIdentifier());
                if (args.get("id").equals(getJSONIdentifier())){
                    if (args.containsKey("add")){

                        JSONObject jadd = (JSONObject)args.get("add");
                        switch ((String) ((JSONObject) jadd.get("id")).get("type")) {
                            case "processor" -> modules.getChildren().add(new Processor(jadd));
                            case "module" -> modules.getChildren().add(new Module(jadd));
                            case "connection" -> connections.getChildren().add(new Connection());
                            default -> System.out.println("[Processor] Unexpected value: " + (String) jadd.get("type"));
                        }
                    }
                }
            }
        }
        if (event.equals("state")){
            /*
            "{
            "id":{...},
            "modules":
                [{"id": {"type": "processor"}, "modules":[{},{},{}], "connections":[{},{},{}], "inputs": [{}], "outputs": [], "config":{}},
                {"type": "module", "id": {}, "inputs": [{}], "outputs": [], "config":{}},
                {2}],
            "connections":[{},{}]
            }"
             */
            if (args.get("id").equals(getJSONIdentifier())){
                System.out.println("[Processor] not implemented load function called!");
            }
        }
    }
}
