package mdd.client;

import javafx.collections.ObservableList;
import javafx.scene.control.Tab;
import javafx.scene.control.TabPane;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import org.json.simple.JSONObject;

import java.util.Vector;


public class Workshop extends TabPane implements IGUIEventClient {
    private GUIEventHandler _eventHandler;
    private Processor _main_processor;

    public Workshop(){
        super();
        _eventHandler = GUIEventHandler.getEventHandler();
        _eventHandler.addEventListener(this);
        loadExample();
        showMainProcessor();
    }

    private void loadExample(){
        _main_processor = new Processor(new Vector<String>(),"0");
        System.out.println("[Workshop] " + _main_processor.getJSONIdentifier().toString());
        _main_processor.addModule(new Module(_main_processor.getAsPrefix(), "1", Color.DODGERBLUE));
        _main_processor.addModule(new Module(_main_processor.getAsPrefix(),"2", Color.DARKRED));

        Processor processor3 = new Processor(_main_processor.getAsPrefix(),"3");

        Processor processor4 = new Processor(processor3.getAsPrefix(),"4");
        processor4.addModule(new Module(processor4.getAsPrefix(),"5", Color.DODGERBLUE));
        processor4.addModule(new Module(processor4.getAsPrefix(),"6", Color.DARKRED));
        processor4.addModule(new Module(processor4.getAsPrefix(),"7", Color.DODGERBLUE));
        processor4.addConnection(new Connection(processor4.getModule("5").getInput("1"), processor4.getModule("6").getOutput("1")));
        processor4.addConnection(new Connection(processor4.getModule("7").getInput("3"), processor4.getModule("5").getOutput("2")));

        processor3.addModule(processor4);

        _main_processor.addModule(processor3);

        _main_processor.addConnection(new Connection(_main_processor.getModule("1").getInput("1"), _main_processor.getModule("3").getOutput("1")));
    }

    public void showMainProcessor(){
        showProcessor(_main_processor);
    }

    private void showProcessor(Processor processor){
        int existingID = -1;
        for (int i = 0; i < getTabs().size(); ++i){
            if (getTabs().get(i).getId().equals(processor.getAsPrefix().toString())){
                existingID = i;
            }
        }
        if (existingID == -1) {
            WorkBench bench = new WorkBench(processor);
            getTabs().add(bench);
            getSelectionModel().select(bench);
        }else{
            getSelectionModel().select(getTabs().get(existingID));
        }
    }


    @Override
    public void close() throws Exception {
        _eventHandler.removeEventListener(this);
    }

    @Override
    public void processGUIEvent(String event, JSONObject args) {

        if (event == "OPEN_PROCESSOR"){
            System.out.println("[Workshop]: OPEN_PROCESSOR with path = " +args.get("path").toString());
            Processor processor = _main_processor.getProcessor((Vector<String>)args.get("path"));
            if (processor != null){
                showProcessor(processor);
            }else{
                System.out.println("[Workshop]: Something went wrong!");
            }

        }
    }
}
